import unittest
from  pyimgren.pyimgren import exif_dat
import datetime
import os

class ExifDatTest(unittest.TestCase):
    def test_exif_dat(self):  # control dat of a JPEG file
        folder = os.path.dirname(__file__)
        self.assertEqual(datetime.datetime(2018, 8, 29, 15, 24, 20),
                         exif_dat(os.path.join(folder, "DSCF9762.JPG")))
        



import unittest
try:
    from unittest import mock
except:
    import mock
import pyimgren
import os.path
import io
import collections
import datetime

class RenamerTest(unittest.TestCase):
    
    @classmethod
    def setUpClass(cls):
        cls.folder = os.path.dirname(__file__)
    def setUp(self):
        self.obj = pyimgren.Renamer(self.folder)
    def test_load_names_ok(self):
        """Extract a name from a names.log file"""
        with unittest.mock.patch("pyimgren.pyimgren.io.open",
                                 return_value = io.StringIO(u"""a:b
c:d
""")):
            names = self.obj.load_names()
            self.assertEqual(2, len(names.keys()))
            self.assertEqual("b", names["a"])
    def test_load_names_ko(self):
        """Try to extract a name from a broken names.log file"""
        with mock.patch("pyimgren.pyimgren.io.open",
                                 return_value = io.StringIO("""a:b
c
:d
""")):
            try:
                names = self.obj.load_names()
            except pyimgren.pyimgren.NamesLogException as e:
                self.assertEqual(2, e.numlig)
                self.assertEqual("c\n", e.line)
                self.assertTrue(isinstance(str(e), str))
                return
            self.fail("No NamesLogException")
    def test_get_names(self):
        """find a new name pretending 55 files already exist"""
        with mock.patch("os.path.exists",
                        side_effect = ([True] * 55) + [False]):
            n = self.obj.get_new_name("foo")
            self.assertEqual(56, os.path.exists.call_count)
            self.assertEqual(os.path.join(self.obj.folder, "foobc")
                             + self.obj.ext_mask,
                             os.path.exists.call_args[0][0])
            self.assertEqual(os.path.join(self.obj.folder, "foo")
                             + self.obj.ext_mask,
                             os.path.exists.call_args_list[0][0][0])
            self.assertEqual(os.path.join(self.obj.folder, "fooa")
                             + self.obj.ext_mask,
                             os.path.exists.call_args_list[1][0][0])
            self.assertEqual("foobc" + self.obj.ext_mask, n)
    def test_back(self):
        """Rename back 3 files"""
        flist = [("a", "b"), ("c", "d"), ("e", "f")]
        with mock.patch("os.rename"),mock.patch.object(
            self.obj, "load_names", return_value =
            collections.OrderedDict(flist)), mock.patch(
                "glob.glob", side_effect = [
                    [ os.path.join(self.folder, i)]
                    for i,j in flist]):
            
            self.obj.back()
            self.assertEqual(3, os.rename.call_count)
            for i, pair in enumerate(flist):
                self.assertEqual(
                    tuple(map(lambda x: os.path.join(self.folder, x),pair)),
                    os.rename.call_args_list[i][0])
    def test_rename(self):
        """Rename 4 files"""
        fd = mock.Mock()
        dates = [
            datetime.datetime(2018,2,1),
            datetime.datetime(2018, 3, 1),
            datetime.datetime(2018, 2, 2),
            datetime.datetime(2018, 4, 1)
            ]
        names = [os.path.join(self.folder, i) for i in "abcd"]
        open_ctx = mock.Mock()
        open_ctx.__enter__ = mock.Mock(return_value = fd)
        open_ctx.__exit__ = mock.Mock()
        with mock.patch("os.rename"), \
             mock.patch.object(self.obj, "load_names",
                               return_value = collections.OrderedDict()), \
             mock.patch("io.open", return_value = open_ctx), \
             mock.patch("glob.glob", return_value = names), \
             mock.patch("pyimgren.pyimgren.exif_dat",
                        side_effect = dates):
            self.obj.rename()
            for i, n in enumerate(names):
                nn0 = dates[i].strftime(self.obj.dst_mask) + self.obj.ext_mask
                nn = os.path.join(self.folder, nn0)
                self.assertEqual((n, nn), os.rename.call_args_list[i][0])
                self.assertEqual(nn0 + ":" +
                                 os.path.relpath(n, self.folder) + "\n",
                                 fd.write.call_args_list[i][0][0])
            
    def test_rename_dir(self):
        """Rename a sub-folder"""
        abs_folder = os.path.join(self.folder, "sub")
        sub = mock.Mock(spec = pyimgren.Renamer)
        with mock.patch("os.path.join", return_value=abs_folder) as join, \
             mock.patch("os.path.isdir") as isdir, \
             mock.patch("glob.glob", return_value=[abs_folder]), \
             mock.patch("pyimgren.pyimgren.Renamer", return_value = sub) as ren:
            self.obj.rename("sub")
            ren.assert_called_once_with(abs_folder, self.obj.src_mask,
                                    self.obj.dst_mask,
                                    self.obj.ext_mask,
                                    self.obj.ref_file,
                                    self.obj.debug,
                                    self.obj.dummy)
            self.assertTrue(sub.log is self.obj.log)
            sub.rename.assert_called_once_with()
            
    def test_unknown_picture(self):
        """Try to rename back a file not known in names.log"""
        log = mock.Mock()
        with mock.patch("os.path.isdir", return_value = False), \
             mock.patch("os.path.relpath", return_value = "xy"), \
             mock.patch("glob.glob", return_value = ["xy"]), \
             mock.patch.object(self.obj, "log") as log, \
             mock.patch.object(self.obj, "load_names",
                return_value = collections.OrderedDict([("a", "b")])):
        
            self.obj.back("xy")
            e = log.method_calls[0][1][0]
            self.assertTrue(isinstance(e,
                            pyimgren.pyimgren.UnknownPictureException))
            self.assertEqual(self.obj.folder, e.folder)
            self.assertEqual(self.obj.ref_file, e.ref_file)
    def test_back_dir(self):
        """Rename back a sub-folder"""
        abs_folder = os.path.join(self.folder, "sub")
        sub = mock.Mock(spec = pyimgren.Renamer)
        with mock.patch("os.path.join", return_value=abs_folder), \
             mock.patch("os.path.isdir", return_value = True), \
             mock.patch("glob.glob", return_value=[abs_folder]), \
             mock.patch("pyimgren.pyimgren.Renamer", return_value = sub) as ren:
            self.obj.back("sub")
            ren.assert_called_once_with(abs_folder, self.obj.src_mask,
                                    self.obj.dst_mask,
                                    self.obj.ext_mask,
                                    self.obj.ref_file,
                                    self.obj.debug,
                                    self.obj.dummy)
            self.assertTrue(sub.log is self.obj.log)
            sub.back.assert_called_once_with()
    def test_too_many_files(self):
        """Try to rename more than 700 files with same timestamp"""
        with mock.patch("os.path.exists", return_value = True):
            try:
                n = self.obj.get_new_name("foo")
            except Exception as e:
                self.assertTrue("foo" in str(e))
                return
        self.fail("No exception")

    def test_inexistant_file(self):
        """Try to rename an inexistant file"""
        with mock.patch.object(self.obj, "log") as log:
            self.obj.rename("foo")
            self.assertEqual("warning", log.method_calls[0][0])

        
class MergeTest(unittest.TestCase):
    """Tests for the merge method"""
    
    @classmethod
    def setUpClass(cls):
        cls.folder = os.path.dirname(__file__)

    def setUp(self):
        self.obj = pyimgren.Renamer(self.folder)

    def test_incorrect_files(self):
        """A file containing a folder should issue a warning and not copy."""
        with mock.patch("pyimgren.pyimgren._copy") as copy, \
             mock.patch("glob.glob", side_effect=lambda x: [x]), \
             mock.patch("os.path.samefile", return_value = False), \
             mock.patch("pyimgren.pyimgren._move") as move, \
             mock.patch.object(self.obj.log, "warning") as warning:
            self.obj.merge("/foo", "fee/bar")
            self.assertEqual(1, warning.call_count)
            move.assert_not_called()
            copy.assert_not_called()

    def test_copy_called(self):
        """merge calls _copy and not _move."""
        with mock.patch("pyimgren.pyimgren._copy") as copy, \
             mock.patch("glob.glob", side_effect=lambda x: [x]), \
             mock.patch("pyimgren.pyimgren._move") as move, \
             mock.patch("pyimgren.pyimgren.exif_dat",
                        return_value=datetime.datetime(2016, 6, 4, 15, 9, 10)):
            self.obj.merge(os.path.join(self.folder, ".."), "foo", "bar")
            self.assertEqual(2, copy.call_count)
            copy.assert_any_call(os.path.join(self.folder, "..", "foo"),
                                          self.folder, "20160604_150910.jpg")
            copy.assert_called_with(os.path.join(self.folder, "..", "bar"),
                                          self.folder, "20160604_150910.jpg")
            move.assert_not_called()
            
    def test_ignore_dir(self):
        """merge should ignore directories and warn."""
        with mock.patch("pyimgren.pyimgren._copy") as copy, \
             mock.patch("glob.glob", side_effect=lambda x: [x]), \
             mock.patch("pyimgren.pyimgren.exif_dat",
                        return_value=datetime.datetime(2016, 6, 4, 15, 9, 10)),\
             mock.patch("os.path.isdir", side_effect= [True, False ]), \
             mock.patch.object(self.obj.log, "warning") as warning:
            self.obj.merge(os.path.join(self.folder, ".."), "foo", "bar")
            self.assertEqual(1, copy.call_count)
            copy.assert_called_once_with(os.path.join(self.folder, "..", "bar"),
                                          self.folder, "20160604_150910.jpg")
            self.assertEqual(1, warning.call_count)
   
    def test_merge_self(self):
        """merge should raise when merging its own folder"""
        self.assertRaises(pyimgren.pyimgren.PyimgrenException,
                          self.obj.merge,
                          self.folder)



from pyfakefs.fake_filesystem_unittest import TestCase
import os.path
import shutil

import sys
parent = os.path.dirname(os.path.dirname(__file__))
if parent not in sys.path:
    sys.path.append(parent)

from pyimgren import Renamer

class SimpleTest(TestCase):
    def setUp(self):
        """Initialize a Renamer in /test (in a fake file system).
Import current directory in /orig to have a known image in DSCF9762.JPG"""
        self.setUpPyfakefs()
        self.fs.add_real_directory(os.path.dirname(__file__),
                                   target_path = "/orig")
        self.fs.create_dir("/test")
        self.ren = Renamer("/test")
        
    def test_orig(self):
        """Control existance of the known jpeg image in /orig"""
        self.assertTrue(os.path.exists("/orig/DSCF9762.JPG"))
        

    def test_simple_rename(self):
        """Rename a single file by giving its name and control:
    * that the original file name has disappared
    * that names.log has received the expected values
"""
        shutil.copyfile("/orig/DSCF9762.JPG", "/test/foo")
        self.ren.rename("foo")
        self.assertFalse(os.path.exists("/test/foo"))
        self.assertTrue(os.path.exists("/test/names.log"))
        with open("/test/names.log") as fd:
            names = self.ren.load_names()
        self.assertEqual(list(names.values()), ["foo"])
        new_name = list(names.keys())[0]
        self.assertTrue(os.path.exists(os.path.join("/test", new_name)))

    def test_simple_back(self):
        """Rename a single file (by its name), rename back and controls that
the file is actually back"""
        shutil.copyfile("/orig/DSCF9762.JPG", "/test/foo")
        self.ren.rename("foo")
        with open("/test/names.log") as fd:
            names = self.ren.load_names()
        new_name = list(names.keys())[0]
        self.ren.back()
        self.assertTrue(os.path.exists("/test/foo"))

    def test_dir_rename(self):
        """Call rename on a subdir and controls that:
   * the main folder was not touched
   * the rename occured in the sub folder"""
        self.fs.create_dir("/test/sub")
        shutil.copyfile("/orig/DSCF9762.JPG", "/test/DSCF9762.JPG")
        shutil.copyfile("/orig/DSCF9762.JPG", "/test/sub/DSCF9762.JPG")
        self.ren.rename("sub")
        self.assertTrue(os.path.exists("/test/DSCF9762.JPG"))
        self.assertFalse(os.path.exists("/test/sub/DSCF9762.JPG"))
        self.assertTrue(os.path.exists("/test/sub/names.log"))
        self.assertFalse(os.path.exists("/test/names.log"))

    def test_dir_back(self):
        """Call rename then back on a subfolder and controls that the image
file is back in the sub folder"""
        self.fs.create_dir("/test/sub")
        shutil.copyfile("/orig/DSCF9762.JPG", "/test/DSCF9762.JPG")
        shutil.copyfile("/orig/DSCF9762.JPG", "/test/sub/DSCF9762.JPG")
        self.ren.rename("sub")
        self.ren.back("sub")
        self.assertTrue(os.path.exists("/test/DSCF9762.JPG"))
        self.assertTrue(os.path.exists("/test/sub/DSCF9762.JPG"))
        self.assertTrue(os.path.exists("/test/sub/names.log"))
        self.assertFalse(os.path.exists("/test/names.log"))


    def test_common_timestamp(self):
        """Rename many files having same timestamp and control the names they
receive"""
        shutil.copyfile("/orig/DSCF9762.JPG", "/test/foo")
        shutil.copyfile("/orig/DSCF9762.JPG", "/test/bar")
        shutil.copyfile("/orig/DSCF9762.JPG", "/test/fee")
        self.ren.rename("foo", "bar")
        self.assertTrue(os.path.exists("/test/fee"))
        self.assertFalse(os.path.exists("/test/foo"))
        self.assertFalse(os.path.exists("/test/bar"))
        names = self.ren.load_names()
        new_names = sorted(names.keys())
        self.assertEqual(new_names[1][:-4], new_names[0][:-4] + "a")


class MergeTest(TestCase):
    """Tests for the merge feature."""

    def setUp(self):
        self.setUpPyfakefs()
        self.fs.add_real_directory(os.path.dirname(__file__),
                                   target_path = "/orig")
        self.fs.create_dir("/test")
        self.ren = Renamer("/test")
        self.fs.create_dir("/src")

    def test_simple_merge(self):
        """Merge a single file by giving its name and control that:
            * the original file not been touched
            * no names.log was created
            * a new file exists in /test
        """
        shutil.copyfile("/orig/DSCF9762.JPG", "/src/foo")
        self.ren.merge("/src", "foo")
        self.assertTrue(os.path.exists("/src/foo"))
        self.assertFalse(os.path.exists("/test/names.log"))
        files = os.listdir("/test")
        self.assertEqual(1, len(files))

    def test_multiple_merge(self):
        """Merge 2 files with the same exif timestamp.

        Controls that second has same base name than first one + 'a'
        """
        shutil.copyfile("/orig/DSCF9762.JPG", "/src/foo")
        shutil.copyfile("/orig/DSCF9762.JPG", "/src/bar")
        self.ren.merge("/src", "foo", "bar")
        files = sorted(os.listdir("/test"))
        self.assertEqual(2, len(files))
        self.assertEqual(files[1][:-4], files[0][:-4] + "a")
    
if __name__ == "__main__":
    import unittest
    unittest.main(verbosity = 2)



import unittest
import pyimgren
import re

class InfraTest(unittest.TestCase):
    """Controls that pyimgren__version is defined, is a string, and is
above 0.0"""
    def test_version_is_string(self):
        self.assertTrue(isinstance(pyimgren.__version__, str),
                        "No version string")
    def test_version_format(self):
        m = re.match(r"(\d+).(\d+).(\d+)", pyimgren.__version__)
        self.assertIsNotNone(m, "Wrong format version")
        majv = int(m.group(1))
        minv = int(m.group(2))
        self.assertTrue(majv > 0 or minv > 0, "Unacceptable version 0.0")



import unittest
from pyimgren import __version__, Renamer
from pyimgren.__main__ import set_parser, main
from unittest.mock import patch, Mock
import pyimgren.__main__
import sys

class TestMain(unittest.TestCase):
    def test_parser_normal(self):
        """Controls decoding of various params"""
        self.parser = set_parser()
        self.parser.prog = "pyimgren"
        args_list = {"-D -f fold back":
                     {"folder": "fold", "src_mask": "DSCF*.jpg",
                      "dst_mask": "%Y%m%d_%H%M%S",
                      "ext_mask": ".jpg", "ref_file": "names.log",
                      "debug": True, "dummy": False, "subcommand": "back",
                      "files": []
                      },
                     "--ext=.jpeg -X -f fold rename":
                     {"folder": "fold", "src_mask": "DSCF*.jpg",
                      "dst_mask": "%Y%m%d_%H%M%S",
                      "ext_mask": ".jpeg", "ref_file": "names.log",
                      "debug": False, "dummy": True, "subcommand": "rename",
                      "files": []
                      },
                     """--src=IMG*.* --dst=%Y%m%d%H%M%S -f fold rename""":
                     {"folder": "fold", "src_mask": "IMG*.*",
                      "dst_mask": "%Y%m%d%H%M%S",
                      "ext_mask": ".jpg", "ref_file": "names.log",
                      "debug": False, "dummy": False, "subcommand": "rename",
                      "files": []
                      },
                     "-r names.txt --folder=fold rename":
                     {"folder": "fold", "src_mask": "DSCF*.jpg",
                      "dst_mask": "%Y%m%d_%H%M%S",
                      "ext_mask": ".jpg", "ref_file": "names.txt",
                      "debug": False, "dummy": False, "subcommand": "rename",
                      "files": []
                      },
                     "-r names.txt -f fold rename foo bar":
                     {"folder": "fold", "src_mask": "DSCF*.jpg",
                      "dst_mask": "%Y%m%d_%H%M%S",
                      "ext_mask": ".jpg", "ref_file": "names.txt",
                      "debug": False, "dummy": False, "subcommand": "rename",
                      "files": ["foo", "bar"]
                      },
                     "merge fold foo bar":
                     {"folder": ".", "src_mask": "DSCF*.jpg",
                      "dst_mask": "%Y%m%d_%H%M%S",
                      "ext_mask": ".jpg", "ref_file": "names.log",
                      "debug": False, "dummy": False, "subcommand": "merge",
                      "files": ["foo", "bar"], "src_folder": "fold"
                      },
                     }
        for (args, v) in args_list.items():
            params = self.parser.parse_args(args.split())
            self.assertIsNotNone(params)
            self.assertEqual(v, vars(params))

    def test_main_rename(self):
        """Controls script call as rename"""
        with patch("pyimgren.__main__.Renamer") as patcher:
            sys.argv = [ "pyimgren", "-D", "-f", "tests", "rename" ]
            ren = patcher("f")
            patcher.side_effect = [ren, Exception("Only one Renamer") ]
            main()
            ren.rename.assert_called_once_with()
            ren.back.assert_not_called()
            call = pyimgren.__main__.Renamer.call_args
        kwargs = call[1]
        self.assertEqual(True, kwargs["debug"])
        self.assertEqual(False, kwargs["dummy"])
        self.assertEqual("tests", kwargs["folder"])
            
    def test_main_back(self):
        """Controls script call as back"""
        with patch("pyimgren.__main__.Renamer") as patcher:
            sys.argv = [ "pyimgren", "-Xf", "foo", "back" ]
            ren = patcher("f")
            patcher.side_effect = [ren, Exception("Only one Renamer") ]
            main()
            ren.back.assert_called_once_with()
            ren.rename.assert_not_called()
            call = pyimgren.__main__.Renamer.call_args
        kwargs = call[1]
        self.assertEqual(False, kwargs["debug"])
        self.assertEqual(True, kwargs["dummy"])
        self.assertEqual("foo", kwargs["folder"])

    def test_main_params(self):
        """Controls passing of script options"""
        with patch("pyimgren.__main__.Renamer") as patcher:
            sys.argv = [ "pyimgren", "-Xf", "foo", "back" ]
            ren = patcher("f")
            patcher.side_effect = [ren, Exception("Only one Renamer") ]
            main()
            call = pyimgren.__main__.Renamer.call_args
        ren = pyimgren.Renamer(*(call[0]), **(call[1]))
        self.assertTrue(ren.dummy)
        self.assertFalse(ren.debug)
        self.assertEqual("foo", ren.folder)

    def test_main_merge(self):
        """Controls script call as merge"""
        with patch("pyimgren.__main__.Renamer") as patcher:
            sys.argv = [ "pyimgren", "-D", "-f", "tests", "merge", "/folder" ]
            ren = patcher("f")
            patcher.side_effect = [ren, Exception("Only one Renamer") ]
            main()
            ren.merge.assert_called_once_with("/folder")
            ren.rename.assert_not_called()
            call = pyimgren.__main__.Renamer.call_args
        kwargs = call[1]
        self.assertEqual(True, kwargs["debug"])
        self.assertEqual(False, kwargs["dummy"])
        self.assertEqual("tests", kwargs["folder"])
        



# MIT License
# Copyright (c) 2018 s-ball

from .pyimgren import Renamer
from .version import version as __version__
# import .pyimgren

__all__ = ["Renamer", "__version__", "__name__"]

import gettext
import locale
import os.path

def nls_init(reset=False):
    if reset:
        pyimgren._ = lambda x: x
    else:
        loc = locale.getlocale()[0]
        if loc is None: loc = locale.getdefaultlocale()[0]

        pyimgren._ = gettext.translation("pyimgren",
                                os.path.join(os.path.dirname(__file__),
                                             "locale"),
                                loc,
                                fallback=True).gettext



# MIT License
# Copyright (c) 2018 s-ball

from . import __version__, Renamer, __name__ as prog
import argparse
import logging
import i18nparse
import gettext
import os.path
import locale
from . import nls_init as ext_init

_ = lambda x: x

def nls_init():
    global _
    localedir = os.path.join(os.path.dirname(__file__), "locale")
    lang = locale.getdefaultlocale()[0]
    tr = gettext.translation("main", localedir, [lang], fallback=True)
    _ = tr.gettext
    ext_init()

def set_parser():
    parser = argparse.ArgumentParser(
        prog = prog,
        description=_("Rename pictures according to their exif timestamp"))
    parser.add_argument("-v", "--version", action="version",
                        version="%(prog)s " + __version__,
                        help=_("show program's version number and exit"))
    parser.add_argument("--folder", "-f", default = ".",
                        help = _("folder containing files to rename"))
    parser.add_argument("-s", "--src_mask", default="DSCF*.jpg",
                        help = _("pattern to select the files to rename"))
    parser.add_argument("-d", "--dst_mask", default="%Y%m%d_%H%M%S",
                        help = _("format for the new file name"))
    parser.add_argument("-e", "--ext", default=".jpg", dest="ext_mask",
                        help = _("extension for the new file name"))
    parser.add_argument("-r", "--ref_file", default="names.log",
                        help = _("a file to remember the old names"))
    parser.add_argument("-D", "--debug", action="store_true",
                        help = _("print a line per rename"))
    parser.add_argument("-X", "--dry_run", action="store_true", dest="dummy",
                        help = _("process normally except no rename occurs"))

    # subcommands configuration (rename, back, merge)
    subparser = parser.add_subparsers(dest='subcommand', help=_("sub-commands"))
    ren = subparser.add_parser("rename", help=
                               _("rename files by using their exif timestamp"))
    ren.add_argument("files", nargs="*",
                      help = _("files to process (default: src_mask)"))
    back = subparser.add_parser("back",
                            help=_("rename files back to their original name"))
    back.add_argument("files", nargs="*",
                    help = _("files to process (default: content of ref_file)"))
    merge = subparser.add_parser("merge",
                                 help=_("merge files from a different folder"))
    merge.add_argument("src_folder", metavar="folder",
                        help = _("folder from where merge picture files"))
    merge.add_argument("files", nargs="*",
                      help = _("files to process (default: src_mask)"))
    # parser.set_defaults(subcommand="rename") # uncomment to have a default
    return parser

def main():
    locale.setlocale(locale.LC_ALL, "")
    i18nparse.activate()
    nls_init()
    parser = set_parser()
    params = parser.parse_args()
    files = params.files
    command = params.subcommand
    kwargs = vars(params)
    del kwargs["subcommand"]
    del kwargs["files"]
    if command == "merge":
        src_folder = params.src_folder
        del kwargs["src_folder"]
    if params.debug:
        log = logging.getLogger("pyimgren")
        log.setLevel(logging.DEBUG)
        log.addHandler(logging.StreamHandler())
    renamer = Renamer(**kwargs)
    if command == "merge":
        renamer.merge(src_folder, *files)
    else:
        getattr(renamer, command)(*files)

if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
from pkg_resources import parse_version
import os.path
import re
import sys

from io import open

NAME = "pyimgren"

# Base version (removes any pre, post, a, b or rc element)
try:
    BASE = get_distribution(NAME).base_version
except:
    BASE = "0.0.0"

# In long description, replace "master" in the build status badges
#  with the current version we are building
with open("README.md") as fd:
    long_description = next(fd).replace("master", BASE)
    long_description += next(fd).replace("latest", BASE)
    long_description += "".join(fd)

setup(
    name=NAME,
    description = "Rename image files according to exif tags",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    packages = find_packages(exclude = ["tests", "docs"]),
    setup_requires = ["setuptools_scm", "mo_installer"],
    use_scm_version = { "write_to": os.path.join(NAME, 'version.py') },
    install_requires = ["piexif", "i18nparse"],
    tests_require = ["pyfakefs"],
    author="s-ball",
    author_email = "s-ball@laposte.net",
    url = "https://github.com/s-ball/pyimgren",
    license = "MIT License",
    project_urls = {
        "Documentation":
            "https://pyimgren.readthedocs.io/en/{}/".format(BASE),
        "Changelog":
            "https://github.com/s-ball/pyimgren/blob/master/CHANGES.txt"
        },
    classifiers = [
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Multimedia :: Graphics",
        ],
    python_requires=">=3",
    test_suite = "tests",
    entry_points = {
        "console_scripts": [
            "pyimgren=pyimgren.__main__:main",
            ],
        },
    package_data = { "": ["locale/*/*/*.mo"]},
    )



"""Implementation of the package"""

# MIT License
# Copyright (c) 2018 s-ball

import os.path
import glob
import datetime
import piexif
import sys
import collections
import io
import logging
import os.path
import shutil

_ = lambda x: x

class PyimgrenException(Exception):
    """Base for pyimgren exceptions"""


class NamesLogException(PyimgrenException):
    """Raised when a line in the names.log file cannot be parsed

    Attributes:
        numlig: line number where the problem occurs
        line  : content of the offending line
    """
    def __init__(self, numlig, line):
        self.numlig = numlig
        self.line = line
    def __str__(self):
        return _("Error in name log file line {}: >{}<").format(
            self.numlig, repr(self.line))


class UnknownPictureException(PyimgrenException):
    """Raised when trying to rename back a file not registered in the
    ref_file

    Attributes:
        file   : name of the unknown image
        renamer: the Renamer object that raised the current error
    """
    def __init__(self, file, renamer):
        self.file = file
        self.ref_file = renamer.ref_file
        self.folder = renamer.folder
    def __str__(self):
        return "File {} in folder {} not found in {}".format(
            self.file, self.folder, self.ref_file)

class MergeSameDirException(PyimgrenException):
    """Raised when trying to merge one directory into itself.

    Attributes:
        folder: name of the directory
        ren: name of the Renamer folder
    """
    def __init__(self, folder):
        self.folder = folder

    def __str__(self):
        return _("Cannot merge {} into itself").format(self.folder)


class Renamer:
    """Main class of the module.

    A Renamer is used to rename image names provided by a camera
    (commonly IMGxxxxx.JPG or DSCFyyyy.JPG into a name based on the time
    when the photography had been taken (as smartphones do). That time is
    extracted from the exif tag of the picture. No rename occurs if the
    picture contains no exif time..

    Parameters:
        folder  : the folder where pictures will be renamed
        src_mask: a pattern to select the files to be renamed (default
                  "DSCF*.jpg")
        dst_mask: a format containing strftime formatting directives, that
                  will be used for the new name of a picture (default
                  "%Y%m%d_%H%M%S")
        ext_mask: the extension of the new name
        ref_file: the name of a file that will remember the old names
                  (default names.log)
        debug   : a boolean flag that will cause a line to be printed for
                  each rename when true
        dummy   : a boolean flag that will cause a "dry run", meaning that
                  the folder will be scanned, and debug info eventually printed
                  but no file will be renamed


    All parameters become attribute of the object with the same name

    Attributes:
        log: an object respecting a logging.Logger interface. By default
            ``logging.getLogger("pyimgren")``

    A file named names.log is created in the folder to store the new names
    and the original ones, in order to be able to rename them back.

    Example::

        conv = Renamer(path)
        conv.rename()   # to convert all files with selected pattern to
                        #  "date" names
        conv.back()

    Note:
        This class requires piexif and Python >= 3.
    """
    
    def __init__(self, folder, src_mask = "DSCF*.JPG",
                 dst_mask = "%Y%m%d_%H%M%S",
                 ext_mask = ".jpg",
                 ref_file = "names.log",
                 debug = False,
                 dummy = False):
        self.folder, self.src_mask, self.dst_mask, self.ref_file = (
            folder, src_mask, dst_mask, ref_file)
        self.ext_mask, self.debug, self.dummy = ext_mask, debug, dummy
        self.log = logging.getLogger("pyimgren")
        
    def rename(self, *pictures):
        """Rename pictures in folder (by default the pictures with the
        src_mask pattern)

        Parameters:
            pictures: an iterable of paths. If a path is a folder, a new Renamer
                is started in that folder with current parameters to rename all
                files matching src_mask. If it is a file (that must be in the
                Renamer folder), that file will be renamed regardless of
                src_mask. If it contains wildcard characters (* and ?), all
                files matching that pattern will berenamed.

        Uses load_names to load the names.log file, and get_new_name to avoid
        collisions in file names.

        Raises:
            RuntimeErrorException:
                if all files from a to zz already exist
        """
        names = self.load_names()
        self._process(names, pictures, self.folder, _move, _subdir)
        if len(names) != 0 and not self.dummy:
            with io.open(
                os.path.join(self.folder, self.ref_file),
                "w", encoding="utf-8") as fd:
                for name, old in names.items():
                    fd.write("{}:{}\n".format(os.path.normcase(name), old))

    def back(self, *pictures):
        """Rename pictures back to their initial name in folder
        (by default all pictures known in ref file)

        Parameters:
            pictures: an iterable of names. If one name exists in the local,
                ref_file, that file will be renamed back. If it contains
                wildcard characters (* and ?), all files matching that
                pattern will be processed.

        Uses load_names to load the names.log file.
        """
        names = self.load_names()
        if len(pictures) == 0:
            files = [os.path.join(self.folder, i) for i in names.keys()]
        else:
            def genfiles():
                for name in pictures:
                    for file in glob.glob(os.path.join(self.folder, name)):
                        yield file
            files = genfiles()
        for file in files:
            if os.path.isdir(file):
                sub = Renamer(file, self.src_mask,
                              self.dst_mask,
                              self.ext_mask,
                              self.ref_file,
                              self.debug,
                              self.dummy)
                sub.log = self.log
                sub.back()
            else:  # it is a file: must be in names
                rel = os.path.relpath(file, self.folder)
                try:
                    orig = names[os.path.normcase(rel)]
                except KeyError as e:
                    self.log.warning(UnknownPictureException(rel,self))
                    continue
                if self.debug: self.log.debug("%s -> %s", rel, orig)
                if not self.dummy:
                    os.rename(file, os.path.join(self.folder, orig))
                    
    def merge(self, src_folder, *files):
        """Merge files from a different folder.

        Parameters:
            src_folder: the name of the folder containing the files to merge.
                It cannot contain wildcard characters.
            *files: file names or patterns containing wilcard characters (* or?)
                defining the files to be copied.

        The files are not moved but remain in their original folder. As usual,
        the copies receive a name based on their exif timestamp encoded by
        strftime using dst_mask and dst_ext.

        If a name matches the directory, the directory is ignored and a warning
        is issued.
        
        Raises:
            RuntimeErrorException:
                if all files from a to zz already exist
        """
        if os.path.samefile(self.folder, src_folder):
            raise MergeSameDirException(self.folder)
        names = collections.OrderedDict()
        self._process(names, files, src_folder, _copy, _warndir)
    
    def load_names(self):
        """Load new and original names from a names.log file.

        Returns:
            OrderedDict:
                the keys of the dict are the new names of the
                renamed pictures and the values are the original names

        Raises:
            NamesLogException:
                the attributes of the NamesLogException are the number
                of the offending line and its content if a line in
                names.log contains no colon (:)
        """
        names = collections.OrderedDict()
        numlig = 0
        try:
            with io.open(os.path.join(self.folder, self.ref_file),
                         encoding="utf-8") as fd:
                for line in fd:
                    numlig += 1
                    row = [i.strip() for i in line.split(":")[:2]]
                    names[row[0]] = row[1]
        except FileNotFoundError:
            pass
        except IndexError as e:
            raise NamesLogException(numlig,line).with_traceback(
                sys.exc_info()[2]) from e
        return names

    def get_new_name(self, name):
        """Finds the final name of a picture if a file with that name
            already exists.

        Parameters:
            name: the name (without the extension which shall be ext_mask)

        Returns:
            str:
                a name composed with
                    * the value of name
                    * a suffix between a and zz until a file of that name does
                      not exist in the directory
                    * ext_mask

        Raises:
            RuntimeErrorException:
                if all files from a to zz already exist
        """        
        if os.path.exists(os.path.join(self.folder, name) + self.ext_mask):
            for i in range(ord("a"), ord("z") + 1):
                n = name + chr(i) + self.ext_mask
                if not os.path.exists(os.path.join(self.folder, n)):
                    return n
            for i in range(ord("a"), ord("z") + 1):
                for j in range(ord("a"), ord("z") + 1):
                    n = name + chr(i) + chr(j) + self.ext_mask
                    if not os.path.exists(os.path.join(self.folder, n)):
                          return n
            raise RuntimeError(_("Too many files for {}").format(
                name + self.ext_mask))
        return name + self.ext_mask

    def _process(self, names, pictures, src_folder, file_action, dir_action):
        """Processing common to rename and merge."""
        if len(pictures) == 0:
            pictures = [self.src_mask]
        for pict in pictures:
            files = glob.glob(os.path.join(src_folder, pict))
            if len(files) == 0:
                self.log.warning(_("{} not found").format(pict))
            else:
                for file in files:
                    if os.path.isdir(file):
                        dir_action(self, file)
                    else:  # it is a file: must be in folder
                        rel = os.path.relpath(file, src_folder)
                        if rel.startswith(".."):
                            self.log.warning(_("%s is not in %s"), file,
                                           src_folder)
                            continue
                        if os.path.dirname(rel) != "":
                            self.log.warning(_("%s is not directly in %s"),
                                           file, src_folder)
                            continue
                            
                        dat = exif_dat(file)
                        if dat is not None:
                            new_name = self.get_new_name(
                                dat.strftime(self.dst_mask))
                            if self.debug:
                                self.log.debug("%s -> %s", rel, new_name)
                            names[new_name] = rel
                            if not self.dummy:
                                file_action(file, self.folder, new_name)
        return names

def _move(file, folder, new_name):
    """Simply rename a file (full path) in a directory (folder)."""
    os.rename(file, os.path.join(folder, new_name))

def _subdir(ren, file):
    """Start a copy of a Renamer in a new folder (file)."""
    sub = Renamer(file, ren.src_mask,
                  ren.dst_mask,
                  ren.ext_mask,
                  ren.ref_file,
                  ren.debug,
                  ren.dummy)
    sub.log = ren.log
    sub.rename()

def _copy(file, folder, new_name):
    shutil.copy(file, os.path.join(folder, new_name))

def _warndir(ren, file):
    ren.log.warning(_("Merge cannot process {}: is a directory"), file)

def exif_dat(file):
    """Extract the timestamp of a picture file from the exif tags.

    This function is a wrapper around the piexif module to easily find the
    date and time when the picture was taken. It first tries the time when
    the camera took the picture, then the time when the file was writen on
    the memory card.
    
    Parameters:
        file: the name of an image file that shall contain an exif tag

    Returns:
        datetime.datetime:
            the date when the picture was taken or stored by the camera found
            in the exif tag or None if no date could be found.
    """
    try:
        exif = piexif.load(file)["Exif"]
    except ValueError:
        return None
    dt = None
    for i in (0x9003, 0x9004):
        dt = exif[i]
        if dt is not None: break
    if dt is None: return None
    return datetime.datetime.strptime(dt.decode("ascii"),
                                      "%Y:%m:%d %H:%M:%S")
